#[macro_use]
extern crate structopt;
#[macro_use]
extern crate failure;

use std::{fs, path::{Path, PathBuf}};
use structopt::StructOpt;

#[derive(Fail, Debug)]
#[fail(display = "path is empty")]
struct PathEmptyError;

fn main() -> Result<(), failure::Error> {
    let app = Yank::from_args();

    let dir = Path::new("/tmp/rcp/");
    if !dir.exists() {
        fs::create_dir(dir)?;
    }

    if app.from.is_some() && app.to.is_some() {
        let app_from = app.from.unwrap();
        let to = app.to.unwrap();
        let from = app_from.as_path();
        let name = from.file_name().ok_or(PathEmptyError)?;
        if to.is_dir() {
            fs::copy(from, to.join(name))?;
        } else {
            fs::copy(from, to)?;
        }
    } else if app.from.is_some() {
        let app_from = app.from.unwrap();
        let from = app_from.as_path();
        let name = from.file_name().ok_or(PathEmptyError)?;
        fs::copy(from, dir.join(name))?;
    } else if app.paste.is_some() {
        let paths = fs::read_dir("/tmp/rcp/")?;
        let cur_dir = Path::new(".");
        let mut fail = false;

        for path in paths {
            let path = path?;
            let file_name = path.file_name();
            let path = path.path();
            let new_file = cur_dir.join(file_name);

            if !fail {
                if let Err(_) = fs::rename(&path, &new_file) {
                    fail = true;
                }
            }
            if fail {
                fs::copy(&path, &new_file)?;
                fs::remove_file(path)?;
            }
        }
    } else {
        eprintln!("No action selected. Run with --help for help.")
    }
    Ok(())
}

#[derive(StructOpt)]
#[structopt(name = "yank", about = "A file copy utility, written in Rust")]
struct Yank {
    #[structopt(help = "A file or directory to copy", parse(from_os_str))]
    from: Option<PathBuf>,
    #[structopt(help = "The directory where the file or directory should be placed", parse(from_os_str))]
    to: Option<PathBuf>,
    #[structopt(short = "p", help = "Moves all the files from the copy sink to the given directory")]
    paste: Option<bool>
}
